* ======= Set of subroutines to be used for tetrahedron method =========

******************* SUBROUTINE BZINTS **********************************
*
      SUBROUTINE BZINTS(JOB,FERWE,CELEN,WTKPT,NBAND,NBANDD,NKPTS,IDTET,
     C               NTET,VOLWGT,EMIN,EMAX,DOS,DOSI,NEDOS,EFERMI,SUMWEI,
     C                     SUME,IU6,PAR,DOSPAR,NKDIM,LDIMP,NIONS,JOBPAR)
*
************************************************************************
*
* This routine performs BZ-integrations by the tetrahedron method. It
* has two basic job modes (controlled by flag 'JOB'):
*    JOB=0:         Calculate the DOS and the integrated DOS
*    JOB=-2,-1,1,2: Calculate the Fermi-weights etc., here you have
*                   for submodes: JOB<0 = without Bloechl-correction,
*                   JOB>0 = with Bloechl-correction, for JOB=+/-2 some
*                   additional output is provided (band energy, number
*                   of electrons, Fermi-energy, Bloechl-correction ...)
*
* Input-parameters are CELEN: the band structure data (epsilon_i,ik)
*                      WTKPT: the weights of the k-points
*                      NBAND: the number of bands
*                      NKPTS: the number of irreducible k-points
*                      IDTET: weights/'coordinates' of all tetrahedra
*                      NTET:  number of tetrahedra
*                      EMAX,EMIN: energy window for DOS/DOSI (JOB=0)
*                      NEDOS: number of energy points for DOS/DOSI
*                      EFERMI: approximate/exact Fermi energy (JOB.NE.0)
*                      IU6: I/O-unit where to write data ...
* Output quantities FERWE:  the Fermi weights for each state (JOB.NE.0)
*                   DOS:    the density of states (JOB=0)
*                   DOSI:   the integrated density of states (JOB=0)
*                   SUMWEI: the sum of all Fermi weights (JOB.NE.0)
*                   SUME:   eigenvalue sum ['band energy'] (JOB.NE.0)
*
************************************************************************

      IMPLICIT COMPLEX (C)

      DIMENSION FERWE(NBANDD,NKPTS),CELEN(NBANDD,NKPTS),IDTET(0:4,NTET)
      DIMENSION DOS(NEDOS),DOSI(NEDOS),EC(4),WC(4,2),WTKPT(NKPTS),IQ(4)
      DIMENSION PAR(NBANDD,NKDIM,LDIMP,NIONS),DOSPAR(NEDOS,LDIMP,NIONS)

* Fatal ERROR!
      IF (NKPTS.LT.4) CALL ERROR(' BZINTS',
     C        ' Tetrahedron method fails (number of k-points < 4)',NKPT)
      IF ((JOB.LT.(-2)).OR.(JOB.GT.2)) CALL ERROR(' BZINTS',
     C  ' JOB must be +/-1 or +/-2 (make weights) or 0 (make DOS)!',JOB)
* Initialize arrays for DOS/integrated DOS (if JOB=0) ...
      IF (JOB.EQ.0) THEN
         DO 1 I=1,NEDOS
            DOS(I)=0.
    1    DOSI(I)=0.
      ELSE
* ... Fermi weights ...
         DO 2 I=1,NBAND
          DO 2 IK=1,NKPTS
    2    FERWE(I,IK)=0.
      END IF
* ... and eigenvalue sums:
      SEV1=0.
      SEV2=0.
* Start looping over tetrahedra:
      DO 20 ITET=1,NTET
* Give a sign of life for long jobs ... :-)
       IF ((NEDOS.GT.1000).AND.(JOBPAR.NE.0)) WRITE(*,'(A,$)') '.'
* Get the four corner points:
       IQ(1)=IDTET(1,ITET)
       IQ(2)=IDTET(2,ITET)
       IQ(3)=IDTET(3,ITET)
       IQ(4)=IDTET(4,ITET)
       DO 20 I=1,NBAND
* Get the eigenvalues at each corner:
         EC(1)=REAL(CELEN(I,IQ(1)))
         EC(2)=REAL(CELEN(I,IQ(2)))
         EC(3)=REAL(CELEN(I,IQ(3)))
         EC(4)=REAL(CELEN(I,IQ(4)))
         IF (JOB.EQ.0) THEN
* Make the DOS:
            CALL SLINZ(VOLWGT*IDTET(0,ITET)*2,EC,EMIN,EMAX,DOS,DOSI,
     C            NEDOS,IQ,PAR,DOSPAR,NKDIM,LDIMP,NIONS,NBANDD,I,JOBPAR)
         ELSE
* Make the weights:
            CALL FSWGTS(VOLWGT*IDTET(0,ITET),EC,EFERMI,WC,(JOB.GT.0))
* Band occupations, band energy and number of electrons ... :
            DO 10 IC=1,4
               SEV1=SEV1+WC(IC,1)*EC(IC)*2
               SEV2=SEV2+WC(IC,2)*EC(IC)*2
               SUMWEI=(WC(IC,1)+WC(IC,2))/WTKPT(IQ(IC))
               FERWE(I,IQ(IC))=FERWE(I,IQ(IC))+SUMWEI
   10       CONTINUE
         END IF
   20 CONTINUE
      IF ((NEDOS.GT.1000).AND.(JOBPAR.NE.0)) WRITE(*,'(A)') ' '
* If JOB=+/-2 make additional checks and give some output (if desired):
      IF (ABS(JOB).EQ.2) THEN
         SUME=SEV1+SEV2
* Check the sum of occupation numbers:
         SUMWEI=0.
         DO 30 I=1,NBAND
          DO 30 IK=1,NKPTS
   30    SUMWEI=SUMWEI+FERWE(I,IK)*WTKPT(IK)
         IF ((IU6.GE.0).AND.(IU6.LE.99))
     C                           WRITE(IU6,40) EFERMI,2*SUMWEI,SUME,SEV2
      END IF
   40 FORMAT(1X,'BZINTS: Fermi energy:',F10.6,';',F10.6,' electrons'/
     C       9X,'Band energy:',F11.6,';  BLOECHL correction:',F10.6)
      RETURN
      END


******************* SUBROUTINE SLINZ ***********************************
*
      SUBROUTINE SLINZ(VOLWGT,EC,EMIN,EMAX,DOS,DOSI,NEDOS,IQ,
     C                PAR,DOSPAR,NKDIM,LDIMP,NIONS,NBANDD,ISTATE,JOBPAR)
*
************************************************************************
*
* This subroutine adds up the contributions to the density of
* states and the number of states for one single tetrahedron
*
*  Input-parameters are VOLWGT: weight on tetrahedron
*                       EC: energies at corners of tetrahedron
*                       EMIN,EMAX: energy window
*                       NEDOS: number of energy points for DOS/DOSI
*  Output quantities DOS(K): DOS at E(K)=EMIN+(K-1)(EMAX-EMIN)/(NEDOS-1)
*                    DOSI(K): Integrated DOS at E(K)
*
************************************************************************

      IMPLICIT REAL (C)

      DIMENSION EC(4),DOS(NEDOS),DOSI(NEDOS),ES(4),EC1(4),IQ(4)
      DIMENSION PAR(NBANDD,NKDIM,LDIMP,NIONS),DOSPAR(NEDOS,LDIMP,NIONS)

      DE=(EMAX-EMIN)/FLOAT(NEDOS-1)
* Sort the energies at the four corners (array EC) into array ES
      DO 1 I=1,4
    1 EC1(I)=EC(I)
      DO 3 I=1,4
         I00=1
         DO 2 J=2,4
    2    IF (EC1(J).LT.EC1(I00)) I00=J
         ES(I)=EC1(I00)
         EC1(I00)=1.E30
    3 CONTINUE
* Lowest energy still above EMAX ---> no contributions to DOS/DOSI ... :
      IF (ES(1).GE.(EMAX+0.00000001*DE)) RETURN
* Highest energy still below EMIN ---> no contribution to DOS and
* contribution of complete tetrahedron to DOSI (1*VOLWGT) ... :
      IF (ES(4).LE.(EMIN-0.00000001*DE)) THEN
         DO 4 I=1,NEDOS
            DOSI(I)=DOSI(I)+VOLWGT
    4    CONTINUE
         RETURN
      END IF
* Now the rest ...
      E1=ES(1)
      E2=ES(2)
      E3=ES(3)
      E4=ES(4)
* Now get the minimum and maximum index for the range we have to update
* DOS(I) and DOSI(I) [so that EMIN>E(ISTART) and EMAX<E(ISTOP)] ... :
      ISTART=MAX((INT((E1-EMIN)/DE-0.00000001)),1)
      ISTART=MIN(ISTART,NEDOS)
      ISTOP=MIN((INT((E4-EMIN)/DE+0.00000001))+2,NEDOS)
      ISTOP=MAX(ISTOP,1)
* Some constants occuring in the integration formulas ... :
      IF ((E3-E2).GT.0.) THEN
         C3=VOLWGT*(E1+E2-E3-E4)/((E3-E1)*(E4-E1)*(E3-E2)*(E4-E2))
         C2=VOLWGT*3./((E3-E1)*(E4-E1))
      ELSE
         C3=0.
         C2=0.
      ENDIF
      C1=C2*(E2-E1)
      C0=C1*(E2-E1)/3.
      IF ((E2-E1).GT.0.) THEN
         CC12=VOLWGT/((E2-E1)*(E3-E1)*(E4-E1))
      ELSE
         CC12=0.
      ENDIF
      IF ((E4-E3).GT.0.) THEN
         CC34=VOLWGT/((E3-E4)*(E2-E4)*(E1-E4))
      ELSE
         CC34=0.
      ENDIF
      DO 7 I=ISTART,ISTOP
         EACT=EMIN+(I-1)*DE
         ADDDOS=0.
* Case EACT between E2,E3:
         IF ((E2.LT.EACT).AND.(EACT.LE.E3)) THEN
            X=EACT-E2
            ADDDOS=C1+X*(2.*C2+3.*X*C3)
            DOSI(I)=DOSI(I)+C0+X*(C1+X*(C2+X*C3))
* Case EACT between E1,E2:
         ELSE IF ((E1.LT.EACT).AND.(EACT.LE.E2)) THEN
            X=EACT-E1
            ADDDOS=3.*CC12*X*X
            DOSI(I)=DOSI(I)+CC12*X*X*X
* Case EACT between E3,E4:
         ELSE IF ((E3.LT.EACT).AND.(EACT.LE.E4)) THEN
            X=EACT-E4
            ADDDOS=-3.*CC34*X*X
            DOSI(I)=DOSI(I)+VOLWGT-CC34*X*X*X
* Case EACT greater than E4 (might probably happen for I=ISTOP):
         ELSE IF (E4.LE.EACT) THEN
            DOSI(I)=DOSI(I)+VOLWGT
         END IF
         DOS(I)=DOS(I)+ADDDOS
         IF (JOBPAR.NE.0) THEN
            DO 6 NI=1,NIONS
             DO 6 LP=1,LDIMP
               PARWGT=0.
               DO 5 IC=1,4
                  PARWGT=PARWGT+PAR(ISTATE,IQ(IC),LP,NI)
    5          CONTINUE
               DOSPAR(I,LP,NI)=DOSPAR(I,LP,NI)+PARWGT*0.25*ADDDOS
    6       CONTINUE
         ENDIF
    7 CONTINUE
* All energies higer than E(ISTOP) give same contribution to DOSI as
* in the case EACT greater than E4 above ... :
      IF (ISTOP.LT.NEDOS) THEN
         DO 10 I=ISTOP+1,NEDOS
            DOSI(I)=DOSI(I)+VOLWGT
   10    CONTINUE
      END IF
      RETURN
      END


******************* SUBROUTINE FSWGTS **********************************
*
      SUBROUTINE FSWGTS(VOLWGT,EC,EFERMI,W,BLOECH)
*
************************************************************************
*
* This routine makes the weights for integration up to EFERMI for
* one single tetrahedron ('contributions to occupation numbers').
*
*  Input-parameters are VOLWGT: weight on tetrahedron
*                       EC: energies at corners of tetrahedron
*                       EFERMI: Fermi energy
*  Output quantities W(I,1): 'normal' weights
*                    W(I,2): Bloechl-corrections to the weights
*
************************************************************************

      IMPLICIT REAL (C)

      LOGICAL BLOECH
      DIMENSION EC(4),EC1(4),ES(4),W(4,2),W1(4),W2(4),ISORT(4)

* Sort the energies at the four corners (array EC) into array ES
      DO 1 I=1,4
    1 EC1(I)=EC(I)
      DO 3 I=1,4
         I00=1
         DO 2 J=2,4
    2    IF (EC1(J).LT.EC1(I00)) I00=J
         ES(I)=EC1(I00)
         ISORT(I)=I00
         EC1(I00)=1.E30
    3 CONTINUE
* Initialise weights and some other things:
      DF=0.
      DO 4 I=1,4
         W1(I)=0.
         W2(I)=0.
         W(I,1)=0.
    4 W(I,2)=0.
* Each k-point belongs to four tetrahedra (and will be touched 4 times
* when looping over all tetrahedra -> 'redefine weight factor' -> VW4):
      VW4=VOLWGT/4.
* Lowest energy still .GE.EFERMI ---> no contributions to weights ... :
      IF (ES(1).GE.EFERMI) RETURN
* Highest energy still .LE.EFERMI ---> just add up full weight (VW4):
      IF (ES(4).LE.EFERMI) THEN
         DO 5 I=1,4
    5    W(I,1)=VW4
         RETURN
      END IF
* Now the rest ... :
      E1=ES(1)
      E2=ES(2)
      E3=ES(3)
      E4=ES(4)
* Case EFERMI between E2,E3:
      IF ((E2.LT.EFERMI).AND.(EFERMI.LE.E3)) THEN
         A31=(EFERMI-E1)/(E3-E1)
         A41=(EFERMI-E1)/(E4-E1)
         A32=(EFERMI-E2)/(E3-E2)
         A42=(EFERMI-E2)/(E4-E2)
         V1=A31*A41
         V2=A31*A42*(1.-A41)
         V3=A42*A32*(1.-A31)
         W1(1)=(V1*(3.-A31-A41)+V2*(2.-A31-A41)+V3*(1.-A31))*VW4
         W1(2)=(V1+V2*(2.-A42)+V3*(3.-A32-A42))*VW4
         W1(3)=(V1*A31+V2*A31+V3*(A31+A32))*VW4
         W1(4)=(V1*A41+V2*(A41+A42)+V3*A42)*VW4
         DF=((E1+E2-E3-E4)*A32*A42+2*EFERMI-E1-E2)/((E3-E1)*(E4-E1))
         DF=3.*VOLWGT*DF
* Case EFERMI between E1,E2:
      ELSE IF ((E1.LT.EFERMI).AND.(EFERMI.LE.E2)) THEN
         A21=(EFERMI-E1)/(E2-E1)
         A31=(EFERMI-E1)/(E3-E1)
         A41=(EFERMI-E1)/(E4-E1)
         XXX=A21*A31*A41*VW4
         W1(1)=XXX*(4.-A21-A31-A41)
         W1(2)=XXX*A21
         W1(3)=XXX*A31
         W1(4)=XXX*A41
         DF=3.*VOLWGT*A31*A41/(E2-E1)
* Case EFERMI between E3,E4:
      ELSE IF ((E3.LT.EFERMI).AND.(EFERMI.LE.E4)) THEN
         A14=(EFERMI-E4)/(E1-E4)
         A24=(EFERMI-E4)/(E2-E4)
         A34=(EFERMI-E4)/(E3-E4)
         XXX=A14*A24*A34*VW4
         W1(1)=VW4-XXX*A14
         W1(2)=VW4-XXX*A24
         W1(3)=VW4-XXX*A34
         W1(4)=VW4-XXX*(4.-A14-A24-A34)
         DF=3.*VOLWGT*A14*A24/(E4-E3)
      END IF
* Here the BLOECHL corrections to the weights (if desired) ... :
      IF (BLOECH) THEN
         DO 15 I=1,4
            W2(I)=0.
            DO 10 J=1,4
   10       W2(I)=W2(I)+(ES(J)-ES(I))*DF*0.025
   15    CONTINUE
      END IF
* Now store the weights into W with the correct ordering ... :
      DO 20 I=1,4
         J=ISORT(I)
         W(J,1)=W1(I)
         W(J,2)=W2(I)
   20 CONTINUE
      RETURN
      END
